// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.cards.
// 
// smithers.cards is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.cards is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.cards. If not, see <http://www.gnu.org/licenses/>.

package smithers.cards;

/**
 * Implentations of this interface specify rules for building on the stack.
 * Such constructs are common in solitaire games, where various rules apply to
 * moving cards around the tableau, foundations, and so on.
 */
public interface BuildingRule {
    
    /**
     * Determines whether the specified card can be added to the top of the
     * stack.
     * 
     * @param stack the stack to add the card to
     * @param add the card to add
     * @return {@code true} iff the card can be added
     */
    public boolean canAddCard(CardStack stack, Card add);
    
    /**
     * Determines whether cards can be moved from one stack to another. This
     * method should return the smallest non-zero number of cards which can be
     * moved (usually this will be the only number of cards which can be
     * moved), or {@code 0} if no cards can be moved. Note this method makes no
     * checks according to whether the required cards can all legally be moved
     * from the source stack, as this stack may be operating under different
     * rules.
     * 
     * @param stack the stack to move cards to
     * @param add the stack to move cards from
     * @return the number of cards which can be moved
     */
    public int canAddCards(CardStack stack, CardStack add);
    
}
