// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.cards.
// 
// smithers.cards is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.cards is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.cards. If not, see <http://www.gnu.org/licenses/>.

package smithers.cards;

/**
 * A building rule which allows a card to be placed if it is the same suit as,
 * and one rank higher than, the top card. Only aces may be placed in an empty
 * space.
 */
public class BuildingUpBySuits extends AbstractBuildingRule {
    
    /**
     * Constructs a new instance of this rule.
     */
    public BuildingUpBySuits() {
        canAddMultiple = false;
    }
    
    /**
     * {@inheritDoc}
     */
    public boolean goesOn(Card c1, Card c2) {
        return c2.sameSuit(c1) && c2.compareRank(c1) == 1;
    }
    
    /**
     * {@inheritDoc}
     */
    public boolean goesInSpace(Card c) {
        return c.getRank() == 1;
    }
    
}
