// Copyright 2010 Christopher Smith
// 
// This file is part of smithers.logicpuzzles.
// 
// smithers.logicpuzzles is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
// 
// smithers.logicpuzzles is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.logicpuzzles. If not, see <http://www.gnu.org/licenses/>.

package smithers.logicpuzzles;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

/**
 * A {@code Component} backed by a logic puzzle.
 */
public class PuzzleComponent extends Component {
    
    private PuzzleGraphics graphics;
    
    /**
     * Create a new {@code PuzzleComponent} which will use the specified
     * {@code PuzzleGraphics} to draw on it.
     * 
     * @param graphics the object which will draw this component's graphics
     */
    public PuzzleComponent(PuzzleGraphics graphics) {
        this.graphics = graphics;
        addMouseListener(graphics);
    }
    
    public void paint(Graphics g) {
        Dimension d = getSize();
        BufferedImage buffer = new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = buffer.createGraphics();
        g2.setClip(g.getClip());
        graphics.draw(g2, new Rectangle(getSize()));
        g.drawImage(buffer, 0, 0, null);
    }
    
    /**
     * Writes an image containing the clues and state of the puzzle.
     * 
     * @param maxSize the size of the grid in output
     * @param file    the file to output to
     */
    public void writeImage(Dimension maxSize, File file) throws IOException {
        BufferedImage result = new BufferedImage(maxSize.width, maxSize.height, BufferedImage.TYPE_INT_RGB);
        Rectangle drawn = graphics.draw(result.createGraphics(), new Rectangle(0, 0, maxSize.width, maxSize.height));
        ImageIO.write(result.getSubimage(drawn.x, drawn.y, drawn.width, drawn.height), "png", file);
    }
    
}
