// Copyright 2010 Christopher Smith
// 
// This file is part of smithers.logicpuzzles.
// 
// smithers.logicpuzzles is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
// 
// smithers.logicpuzzles is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.logicpuzzles. If not, see <http://www.gnu.org/licenses/>.

package smithers.logicpuzzles;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseListener;

/**
 * Interface for graphical representations of logic puzzles. Classes
 * implementing this interface can draw the clues and state of logic puzzles
 * onto graphics objects. They can also handle mouse events, usually using
 * mouse clicks to allow the user to solve the puzzle, or call a solver to
 * solve it. This class may assume that it is only drawing in one place, i.e.
 * when a mouse event is received, the coordinates are the same as in the last
 * call to {@code draw()}.
 */
public interface PuzzleGraphics extends MouseListener {
    
    /**
     * Draws the puzzle onto a graphics object. This method should attempt to
     * fit the puzzle into the given rectangle. The puzzle should be drawn as
     * large as possible and centered in the region.
     * 
     * @param g the graphics object to draw onto
     * @param r the region to fit the puzzle into
     * @return the actual bounding rectangle for the puzzle
     */
    public Rectangle draw(Graphics g, Rectangle r);
    
}
