// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.twisty.
// 
// smithers.twisty is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.twisty is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.twisty. If not, see <http://www.gnu.org/licenses/>.

package smithers.twisty;

/**
 * Extends the {@code TwistyPuzzle} interface to include new methods for
 * animatible twisty puzzles. The units used for timing are entirely up to the
 * whim of the calling class, since the calling class also chooses how much to
 * update the animation by.
 */
public interface AnimatedTwistyPuzzle extends TwistyPuzzle {
    
    /**
     * Twists the puzzle over a specified amount of time. Using a time of 0
     * should apply the twist immediately. Supplying a negative number for the
     * time allows the puzzle to choose the time based on the twist using the
     * absolute value of the time as a maximum.
     * 
     * @param twist the twist to make
     * @param time the time to take
     * @throws IllegalArgumentException if {@code time} is infinite or NaN
     */
    public void twist(Twist twist, double time);
    
    /**
     * Returns true iff the puzzle is partway through a twist. If this method
     * returns true, any modifier in {@code TwistyPuzzle} may fail with an
     * {@code AnimatingException}.
     * 
     * @return {@code true} iff the puzzle is animating a twist
     */
    public boolean isAnimating();
    
    /**
     * Advances the animation by a certain amount of time.
     * 
     * @param time the amount of time to advance
     */
    public void advanceAnimation(double time);
    
}
